"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Surface = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _invariant = _interopRequireDefault(require("invariant"));

var _raf = _interopRequireDefault(require("raf"));

var _glReact = require("gl-react");

var _getContext = _interopRequireDefault(require("./getContext"));

var _loseGL = _interopRequireDefault(require("./loseGL"));

const _excluded = ["width", "height"];

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign ? Object.assign.bind() : function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const propTypes = {
  onContextLost: _propTypes.default.func.isRequired,
  onContextRestored: _propTypes.default.func.isRequired,
  onContextCreate: _propTypes.default.func.isRequired,
  onContextFailure: _propTypes.default.func.isRequired,
  webglContextAttributes: _propTypes.default.object,
  width: _propTypes.default.number.isRequired,
  height: _propTypes.default.number.isRequired
};

class GLView extends _react.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "webglContextAttributes", void 0);

    _defineProperty(this, "canvas", void 0);

    _defineProperty(this, "gl", void 0);

    _defineProperty(this, "onRef", ref => {
      this.canvas = ref;
    });
  }

  componentDidMount() {
    (0, _invariant.default)(this.canvas, "GLView: canvas is not available in componentDidMount!"); // TODO hook context lost / restore

    const {
      webglContextAttributes,
      onContextCreate,
      onContextFailure
    } = this.props;
    const gl = (0, _getContext.default)(this.canvas, webglContextAttributes);
    this.webglContextAttributes = webglContextAttributes || {};

    if (gl) {
      this.gl = gl;
      onContextCreate(gl);
    } else {
      onContextFailure(new Error("no-webgl-context"));
    }
  }

  componentDidUpdate(prevProps) {
    const {
      width,
      height
    } = this.props;

    if (prevProps.width !== width || prevProps.height !== height) {
      if (this.gl) {
        this.gl.getExtension("STACKGL_resize_drawingbuffer").resize(width, height);
      }
    }
  }

  componentWillUnmount() {
    if (this.gl) (0, _loseGL.default)(this.gl);
  }

  simulateContextLost() {
    this.props.onContextLost();
  }

  simulateContextRestored() {
    this.props.onContextRestored(this.gl);
  }

  captureAsDataURL() {}

  captureAsBlob() {}

  render() {
    const _this$props = this.props,
          {
      width,
      height
    } = _this$props,
          rest = _objectWithoutProperties(_this$props, _excluded);

    for (let k in propTypes) {
      delete rest[k];
    }

    return /*#__PURE__*/_react.default.createElement("canvas", _extends({
      ref: this.onRef,
      width: width,
      height: height
    }, rest));
  }

}

_defineProperty(GLView, "propTypes", propTypes);

class RenderLessElement extends _react.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "refMap", new Map());

    _defineProperty(this, "getFirstChild", () => {
      const ref = this.refMap.get(0);
      if (ref && typeof ref.getRootRef === "function") return ref.getRootRef();
      return ref;
    });
  }

  render() {
    const {
      children
    } = this.props;
    return /*#__PURE__*/_react.default.createElement("span", null, _react.default.Children.map(children, (element, i) => {
      if (!element) return element;

      const cloneRef = ref => this.refMap.set(i, ref);

      const originalRef = element.ref;

      if (typeof originalRef === "string") {
        return element; // we don't want to be intrusive
      }

      return /*#__PURE__*/_react.default.cloneElement(element, {
        ref: !originalRef ? cloneRef : component => {
          cloneRef(component);
          originalRef(component);
        }
      });
    }));
  }

}

_defineProperty(RenderLessElement, "propTypes", {
  children: _propTypes.default.any
});

const Surface = (0, _glReact.createSurface)({
  GLView,
  RenderLessElement,
  // $FlowFixMe trust me flow xD
  mapRenderableContent: el => el.getFirstChild(),
  requestFrame: _raf.default,
  cancelFrame: _raf.default.cancel
});
exports.Surface = Surface;
//# sourceMappingURL=index.js.map